// ToolView.cpp : implementation of the CToolView class
//

#include "stdafx.h"
#include "Tool.h"

#include "ToolDoc.h"
#include "ToolView.h"
#include ".\toolview.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif


// CToolView

IMPLEMENT_DYNCREATE(CToolView, CView)

BEGIN_MESSAGE_MAP(CToolView, CView)
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CView::OnFilePrintPreview)
    ON_COMMAND(ID_DRAW_CIRCLE, OnDrawCircle)
	ON_UPDATE_COMMAND_UI(ID_DRAW_CIRCLE, OnUpdateDrawCircle)
	ON_WM_PAINT()
	ON_WM_LBUTTONDOWN()
	ON_WM_MOUSEMOVE()
	ON_WM_LBUTTONUP()
	ON_WM_SIZE()
	ON_COMMAND(ID_DRAW_BLT, OnDrawBlt)
	ON_UPDATE_COMMAND_UI(ID_DRAW_BLT, OnUpdateDrawBlt)
END_MESSAGE_MAP()

// CToolView construction/destruction

CToolView::CToolView()
: m_bCircle(false)
, m_pntStart(200, -200)
, m_szRect(200, -200)
, m_pntHit(0)
, m_bCaptured(false)
, m_bBlt(false)
{
	// TODO: add construction code here
	m_pdcMemory = new CDC;
	m_pBitmap = new CBitmap;

}

CToolView::~CToolView()
{
	delete m_pBitmap;
	delete m_pdcMemory;
}

BOOL CToolView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CView::PreCreateWindow(cs);
}

// CToolView drawing

void CToolView::OnDraw(CDC* pDC)
{
	CToolDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	// TODO: add draw code for native data here
	
     CBrush newBrush(RGB(255,0,0));
     CBrush* oldBrush = pDC->SelectObject(&newBrush);
	 if (m_bCircle) {
		pDC->Ellipse(20, -20, 200, -200);
	 } else {
		pDC->Rectangle(20, -20, 200, -200);
	 }
     pDC->SelectObject(oldBrush);
	 pDC->Ellipse(CRect(m_pntStart, m_szRect));
}


// CToolView printing

BOOL CToolView::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
}

void CToolView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add extra initialization before printing
}

void CToolView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add cleanup after printing
}


// CToolView diagnostics

#ifdef _DEBUG
void CToolView::AssertValid() const
{
	CView::AssertValid();
}

void CToolView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CToolDoc* CToolView::GetDocument() const // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CToolDoc)));
	return (CToolDoc*)m_pDocument;
}
#endif //_DEBUG


// CToolView message handlers

void CToolView::OnDrawCircle()
{
	m_bCircle ^= 1;
	m_pntStart -= CSize(10,-10);
	Invalidate();
}

void CToolView::OnUpdateDrawCircle(CCmdUI *pCmdUI)
{
	// TODO: Add your command update UI handler code here
	pCmdUI->SetCheck(m_bCircle);
}

void CToolView::OnInitialUpdate()
{
	CView::OnInitialUpdate();

	CSize sizeTotal(800, 1050); // 8-by-10.5 inches
    
	// TODO: Add your specialized code here and/or call the base class
	if (m_pdcMemory->GetSafeHdc() == NULL) {
		CClientDC dc(this);
		OnPrepareDC(&dc);
		CRect rectMax(0, 0, sizeTotal.cx, -sizeTotal.cy);
		dc.LPtoDP(&rectMax);
		m_pdcMemory->CreateCompatibleDC(&dc);
		m_pBitmap->CreateCompatibleBitmap(&dc, rectMax.right,
			rectMax.bottom);
		m_pdcMemory->SetMapMode(MM_LOENGLISH);
	}
}

void CToolView::OnPaint()
{
	CPaintDC dc(this); // device context for painting
	// TODO: Add your message handler code here
	// Do not call CView::OnPaint() for painting messages
	OnPrepareDC(&dc);
	if (m_bBlt) {
		CRect rectUpdate;
		dc.GetClipBox(&rectUpdate);
		CBitmap* pOldBitmap = m_pdcMemory->SelectObject(m_pBitmap);
		//select entire client area
		m_pdcMemory->SelectClipRgn(NULL);
		m_pdcMemory->IntersectClipRect(&rectUpdate);
		CBrush backgroundBrush((COLORREF) ::GetSysColor(COLOR_WINDOW));
		CBrush* pOldBrush = m_pdcMemory->SelectObject(&backgroundBrush);
		m_pdcMemory->PatBlt(rectUpdate.left, rectUpdate.top,
			rectUpdate.Width(), rectUpdate.Height(), PATCOPY);
		OnDraw(m_pdcMemory);
		dc.BitBlt(rectUpdate.left, rectUpdate.top,
			rectUpdate.Width(), rectUpdate.Height(),
			m_pdcMemory, rectUpdate.left, rectUpdate.top, SRCCOPY);
		m_pdcMemory->SelectObject(pOldBitmap);
		m_pdcMemory->SelectObject(pOldBrush);
	} else {
		OnDraw(&dc);
	}
}

void CToolView::OnPrepareDC(CDC* pDC, CPrintInfo* pInfo)
{
	// TODO: Add your specialized code here and/or call the base class
	pDC->SetMapMode(MM_LOENGLISH);
	CView::OnPrepareDC(pDC, pInfo);
}

void CToolView::OnLButtonDown(UINT nFlags, CPoint point)
{
	// TODO: Add your message handler code here and/or call default
	CClientDC dc(this);
	OnPrepareDC(&dc);

	CRect rect(m_pntStart, m_szRect);
	dc.LPtoDP(&rect);
	if (rect.PtInRect(point)) {
		CPoint pntLP(point);
		dc.DPtoLP(&pntLP);
		m_pntHit = pntLP;
		m_bCaptured = true;
		SetCapture();
	}
}

void CToolView::InvalidateOldAndNew(CPoint point)
{
  	CClientDC dc(this);
	OnPrepareDC(&dc);
	
	CRect rect(m_pntStart, m_szRect);
	dc.LPtoDP(&rect);
	rect.InflateRect(2, 2);
	InvalidateRect(&rect, FALSE);

	CPoint pointLP(point);
	dc.DPtoLP(&pointLP);
	m_pntStart += pointLP - m_pntHit;

	rect = CRect(m_pntStart, m_szRect);
	dc.LPtoDP(&rect);
	rect.InflateRect(2, 2);
	InvalidateRect(&rect, FALSE);

	m_pntHit = pointLP;
}

void CToolView::OnMouseMove(UINT nFlags, CPoint point)
{
	// TODO: Add your message handler code here and/or call default
  if (m_bCaptured) {
    InvalidateOldAndNew(point);
  }
}

void CToolView::OnLButtonUp(UINT nFlags, CPoint point)
{
  if (m_bCaptured) {	
    InvalidateOldAndNew(point);
    m_bCaptured = false;
    ::ReleaseCapture();
  }
}

void CToolView::OnSize(UINT nType, int cx, int cy)
{
	CView::OnSize(nType, cx, cy);
	// TODO: Add your message handler code here
}

void CToolView::OnDrawBlt()
{
	// TODO: Add your command handler code here
	m_bBlt ^= 1;
}

void CToolView::OnUpdateDrawBlt(CCmdUI *pCmdUI)
{
	// TODO: Add your command update UI handler code here
	pCmdUI->SetCheck(m_bBlt);
}
